
#include "GSMain.h"

void *recv_message( void *arg );
void *send_message( void *arg );

char message[BUFSIZE];

FILE *fp;
pthread_t recv_thread, send_thread;

void error_handling( char *message );

////////////////////////////////////////////////////

void lj_handler( int esd, struct sockaddr_storage *from, socklen_t fromlen )
{
	struct sockaddr_in tmp;
	memcpy( &tmp, from, sizeof(tmp) );

	mjoin_confirm( esd, 1, from, fromlen );
	printf( "Connected TU IP : %s\n", inet_ntoa(tmp.sin_addr));
}

void tg_handler( int esd, struct sockaddr_storage *from, socklen_t fromlen )
{
	mtoken_confirm( esd, 1, from, fromlen );
}

////////////////////////////////////////////////////

void *recv_message(void *arg)
{
	GSMain *gs = (GSMain*)arg;
	int sock = gs->getSock();

	ssize_t nbytes = -1;
	char lmsg[BUFSIZE+16];
	char tmsg[BUFSIZE];
	struct sockaddr_in p_addr;
	socklen_t p_addr_len;

	p_addr_len = sizeof(p_addr);

	while(1) {
		bzero( lmsg, sizeof(lmsg) );
		bzero( tmsg, sizeof(tmsg) );
		nbytes = mrecvmsg( sock, tmsg, BUFSIZE, 0, 
				(struct sockaddr_storage*)&p_addr, &p_addr_len );
		if( nbytes <= 0 )
			break;
		
		printf( "received IP : %s, token msg : %s\n", 
				inet_ntoa(p_addr.sin_addr), tmsg );
		sprintf( lmsg, "%s\t%s", inet_ntoa(p_addr.sin_addr), tmsg );
		gs->setTokenInfo( lmsg );
	}

	return 0;
}


void *send_message(void *arg)
{
	int sock = (int)arg;

	ssize_t nbytes = -1;

	while( !feof(fp) )
	{
		nbytes = fread( message, 1, BUFSIZE, fp );
		msendm( sock, message, nbytes );
		usleep(300);
	}

	mterminate( sock, 0 ); 
	pthread_cancel(recv_thread);

	return 0;
}

///////////////////////////////////////////////////////////

GSMain::GSMain()
{
	setGeometry( 100, 100, 400, 400 );

	this->setCaption( "ECTP(TC-Owner)" );
	v1 = new QVBoxLayout(this);
	h1 = new QHBoxLayout(v1);

	box = new QGroupBox( 2, Horizontal, "Address", this );

	label1 = new QLabel( "TO IP", box );
	to_ip = new QLineEdit(box);

	label1 = new QLabel( "Group IP", box );
	mul_ip = new QLineEdit(box);

	label2 = new QLabel( "Group PORT", box );
	mul_port = new QLineEdit(box);

	label5 = new QLabel( "FILE NAME", box );
	filename = new QLineEdit(box);
	filename->setText("star10.ts");

	p_start = new QPushButton( "Start", box );
	v1->addWidget(box);

	token_box = new QGroupBox( 2, Horizontal, "Token Message", this );
	token_info = new QListBox(token_box);
	v1->addWidget(token_box);

	connect( p_start, SIGNAL(clicked()), this, SLOT(create()) );

	multi_TTL = TTL;
}

GSMain::~GSMain()
{
}

int GSMain::getSock()
{
	return esd;
}

void GSMain::setTokenInfo( char *token_msg )
{
	token_info->insertItem( QString(token_msg), -1 );
	token_info->setEnabled(false);
	token_info->setEnabled(true);
}

void error_handling( char *message )
{
	fputs( message, stderr );
	fputc( '\n', stderr );
	perror( "Error " );
	exit(1);
}

void GSMain::create()
{
	local_ip = to_ip->text().ascii();
	group_ip = mul_ip->text().ascii();
	group_port = mul_port->text().toInt();
	printf( "IP of Local = %s\n", local_ip );
	printf( "IP of Group = %s, Port = %d\n", group_ip, group_port );

	file_name = filename->text().ascii();

	if( !(fp = fopen( file_name, "r" )) )
		error_handling( "Open() error" );
	printf( "read..... %s\n", file_name );

	/* TO address & Group port assignment. */
	memset( &local_addr, 0, sizeof(local_addr) );
	local_addr.sin_family = AF_INET;
	local_addr.sin_addr.s_addr = inet_addr(local_ip);
	local_addr.sin_port = htons(group_port);

	/* Group address & Group port assignment. */
	memset( &multi_addr, 0, sizeof(multi_addr) );
	multi_addr.sin_family = AF_INET;
	multi_addr.sin_addr.s_addr = inet_addr(group_ip);
	multi_addr.sin_port = htons(group_port);

	/* ECTP-3 API - msocket() */
	if( (esd = msocket(PF_INET, ECTP_ROLE_TO)) < 0 )
		error_handling( "msocket() error" );

	/* ECTP-3 API - msetsockopt() */
	struct ectp_callback opt;
	opt.latejoin_handler = lj_handler;
	opt.tokenget_handler = tg_handler;
	msetsockopt( esd, IPPROTO_ECTP, ECTP_OPT_EVENTS, &opt,
				 sizeof(struct ectp_callback) );

	/* ECTP-3 API - mcreate() */
	if( mcreate(esd, (struct sockaddr_storage *) &local_addr, 
				(struct sockaddr_storage *) &multi_addr, 0) < 0 )
		error_handling( "mcreate() error!" );

	p_start->setEnabled(false);

	/* Application Thread */
	pthread_create( &recv_thread, NULL, recv_message, (void*)this );
	pthread_create( &send_thread, NULL, send_message, (void*)esd );
}

int main( int argc, char **argv )
{
	QApplication qApp(argc, argv);
	GSMain *ectp = new GSMain;
	QSize size = ectp->sizeHint();
	ectp->setMinimumSize(size);
	ectp->setMaximumSize(size);

	ectp->show();

	qApp.setMainWidget(ectp);
	return qApp.exec();
}
