
#ifndef	_ECTP_H
#define	_ECTP_H

#ifdef	__cplusplus
extern "C" {
#endif

/* -------------------------------------------------------------------------- *
 * ECTP-3 Socket Options
 * -------------------------------------------------------------------------- */

#define IPPROTO_ECTP	155

enum ectp_optname
{
	ECTP_OPT_TCO,		/* TCO */
	ECTP_OPT_AGN,		/* AGN */
	ECTP_OPT_MSS,		/* MSS */
	ECTP_OPT_FDN,		/* FDN */
	ECTP_OPT_EVENTS,	/* Events */

	/* Options for forward multicast channel */

	ECTP_OPT_FCH_HGT,	/* HGT */
	ECTP_OPT_FCH_RXT,	/* RXT */

	/* Options for backward unicast channel */

	ECTP_OPT_BCH_HGT,	/* HGT */
	ECTP_OPT_BCH_RXT,	/* RXT */
};


/* -------------------------------------------------------------------------- *
 * The role of a node that is taken part in ECTP-3 control tree.
 * -------------------------------------------------------------------------- */

typedef enum
{
	ECTP_ROLE_NONE		= 0,
#define	ECTP_ROLE_NONE	ECTP_ROLE_NONE

	ECTP_ROLE_TO,
#define ECTP_ROLE_TO	ECTP_ROLE_TO

	ECTP_ROLE_LO,
#define	ECTP_ROLE_LO	ECTP_ROLE_LO

	ECTP_ROLE_LE,
#define	ECTP_ROLE_LE	ECTP_ROLE_LE

} ectp_role_t;

#define	ECTP_ROLE_RA	ECTP_ROLE_LO
#define	ECTP_ROLE_TU	ECTP_ROLE_LE


/* ------------------------------------------------------------------------- *
 * Definitions for ECTP-3 User Callback
 * ------------------------------------------------------------------------- */

struct ectp_callback {
	
	/* Late Join Handler */
	void (*latejoin_handler) (int esd, struct sockaddr_storage *from, 
								socklen_t fromlen);

	/* Tree Join Handler */
	/* void (*treejoin_handler) (SS *from, socklen_t *fromlen); */

	/* Token Get Handler */
	void (*tokenget_handler) (int esd, struct sockaddr_storage *from, 
								socklen_t fromlen);
};


/* -------------------------------------------------------------------------- *
 * ECTP-3 Socket APIs Def.
 * -------------------------------------------------------------------------- */

extern int msocket( int domain, int role );

extern int mcreate( int esd, struct sockaddr_storage *local_addr,
				 	struct sockaddr_storage *group_addr, int time );

extern int mjoin( int esd, struct sockaddr_storage *local_addr, 
				  struct sockaddr_storage *group_addr, int time );

extern int mterminate( int esd, int f );

extern int mleave( int esd, const struct sockaddr_storage *to, 
					socklen_t tolen );

extern int mtoken_get( int esd );

extern int mtoken_return( int esd );

extern ssize_t msendm( int esd, const void *msg, size_t len );

extern ssize_t msendmsg( int esd, const void *msg, size_t len, int flags,
				  const struct sockaddr_storage *to, socklen_t tolen );

extern ssize_t mrecvmsg( int esd, void *msg, size_t len, int flags,
						 struct sockaddr_storage *from, socklen_t *fromlen );

extern int msetsockopt( int esd, int level,
						int optname, void *optval, socklen_t optionlen );

extern int mgetsockopt( int esd, int level, int optname,
						void *optval, socklen_t *optionlen );

extern int mjoin_confirm( int esd, int accept,
						  const struct sockaddr_storage *to, socklen_t tolen );

extern int mtoken_confirm( int esd, int accept,
						   const struct sockaddr_storage *to, socklen_t tolen );

#ifdef	__cplusplus
}
#endif

#endif	/* _ECTP_H */
