/* Enhanced Communication Transport Protocol Part-3 Implementation
 *
 * This file is part of the ECTP-3 over UDP Implementation.
 *
 * This header represents the structures and constants needed to support
 * the ECTP-3 to the Sockets API.
 *
 * Please send any bug reports or fixes you make to the email address(es):
 * 	ECTP-3 developers <saintpaul1978@mail.knu.ac.kr>
 *
 * Written or modified by:
 * 	Sang Tae Kim				<saintpaul1978@mail.knu.ac.kr>
 * 	Jae Sung Park				<knucsid@gmail.com>
 *
 */

#include <sys/types.h>
#include <sys/socket.h>



#ifndef	_ECTP_USER_H
#define	_ECTP_USER_H

#ifdef	__cplusplus
extern "C" {
#endif



/* -------------------------------------------------------------------------- *
 * ECTP-3 Socket Options
 * -------------------------------------------------------------------------- */

#define IPPROTO_ECTP	155

enum ectp_optname
{
	ECTP_OPT_TCO,		/* TCO */
	ECTP_OPT_AGN,		/* AGN */
	ECTP_OPT_MSS,		/* MSS */
	ECTP_OPT_FDN,		/* FDN */
	ECTP_OPT_EVENTS,	/* Events */

	/* Options for forward multicast channel */

	ECTP_OPT_FCH_HGT,	/* HGT */
	ECTP_OPT_FCH_RXT,	/* RXT */

	/* Options for backward unicast channel */

	ECTP_OPT_BCH_HGT,	/* HGT */
	ECTP_OPT_BCH_RXT,	/* RXT */
};


/* -------------------------------------------------------------------------- *
 * ECTP-3 Socket APIs Def.
 * -------------------------------------------------------------------------- */

extern int msocket( int domain, int role );

extern int mcreate( int esd, struct sockaddr_storage *local_addr,
				 	struct sockaddr_storage *group_addr, int time );

extern int mjoin( int esd, struct sockaddr_storage *local_addr, 
				  struct sockaddr_storage *group_addr, int time );

extern int mterminate( int esd, int f );

extern int mleave( int esd, const struct sockaddr_storage *to, 
					socklen_t tolen );

extern int mtoken_get( int esd );

extern int mtoken_return( int esd );

extern ssize_t msendm( int esd, const void *msg, size_t len );

extern ssize_t msendmsg( int esd, const void *msg, size_t len, int flags,
				  const struct sockaddr_storage *to, socklen_t tolen );

extern ssize_t mrecvmsg( int esd, void *msg, size_t len, int flags,
						 struct sockaddr_storage *from, socklen_t *fromlen );

extern int msetsockopt( int esd, int level,
						int optname, void *optval, socklen_t optionlen );

extern int mgetsockopt( int esd, int level, int optname,
						void *optval, socklen_t *optionlen );

extern int mjoin_confirm( int esd, int accept,
						  const struct sockaddr_storage *to, socklen_t tolen );

extern int mtoken_confirm( int esd, int accept,
						   const struct sockaddr_storage *to, socklen_t tolen );

#ifdef	__cplusplus
}
#endif

#endif	/* _ECTP_USER_H */
