
#include <stdio.h>
#include <string.h>


#ifndef	_ECTP_WRAPPER_H
#define	_ECTP_WRAPPER_H

#ifdef	__cplusplus
extern "C" {
#endif


/**
 * err_ret() prints error message 'msg' & '\n' to 'stderr' and return 'ret_val'.
 *
 * This macro function support consistent manner that the caller prints some
 * error messages and return a return-value regardless of return type.
 *
 * USAGE:
 * 		int test(int condition, char *msg)
 * 		{
 * 			if (condition < 0)
 * 				err_ret(msg, -1);
 * 			return 0;
 * 		}
 * 		struct _test *create_struct(int condition)
 * 		{
 * 			if (condition < 0)
 * 				err_ret("Error on condition\n", NULL);
 * 			return (struct _test *) malloc(sizeof(struct _test));
 * 		}
 */
#define err_ret(msg, ret_val) ({	\
	fputs(msg "\n", stderr);				\
	return ret_val; })

/**
 * err_msg() just prints error message 'msg' & '\n' to 'stderr'.
 */
#define err_msg(msg) fputs(msg "\n", stderr)

/**
 * err_msg() just prints error message 'msg' to 'stderr' and goto 'label'.
 *
 * USAGE:
 * 		int test()
 * 		{
 * 			int condition = some_operation();
 *			if ( condition < 0 )
 *				err_goto("test() : some_operation() failed.\n", rb_test);
 *			return 0;
 *
 *		rb_test:  // Test Rollback Position
 *			cancel_operation();
 *			return -1;
 *		}
 */
#define err_goto(msg, label) ({		\
	fputs(msg "\n", stderr);				\
	goto label; })

/**
 * err_quit() prints error message 'msg' & '\n' to 'stderr' and exit with 1.
 *
 * USAGE:
 * 		if (some_condition < 0)
 * 			err_quit("Error!");
 */
#define err_quit(msg)				\
	do {							\
		fputs(msg "\n", stderr);	\
		exit(1);					\
	} while (0)


/* IF the machine is IA-32, use following code: */

/**
 * ffs - find first bit set
 * @x: the word to search
 *
 * This is defined the same way as
 * the libc and compiler builtin ffs routines, therefore
 * differs in spirit from the above ffz (man ffs).
 *
 * FROM:
 * 		Linux kernel 2.6.15
 */
static inline int __ffs(int x)
{
	int r;

	__asm__("bsfl %1,%0\n\t"
		"jnz 1f\n\t"
		"movl $-1,%0\n"
		"1:" : "=r" (r) : "rm" (x));
	return r+1;
}

#ifdef	__cplusplus
}
#endif

#endif	/* _ECTP_WRAPPER_H */
